function zeta=ellc3(Ws,Eps)
% ############################################################################
%
% ellc3.m   zeta=ellc3(Ws,Eps) calculates the zeta of the third-order 
%           Cauer filter.
%           Ws  : normalized stopband edge frequency (unity cutoff frequency)
%           Eps : ripple - factor
%           ellc3.m returns zeta. The zeta=ellc3 is a function used to
%           calculate the poles of the Cauer filters of the order 
%           N= 3, 6, 9, 12, 18 (ellc.m).
%
%           M. Lutovac 6-18-1994
%
% function zeta=ellc3(Ws,Eps)
% ____________________________________________________________________________

W=ellz30(Ws);                       Z=Ws/W;
X0=W^2;        Xz=Z^2;              E=Eps^2;
A0=E*(1-Xz)^2;                      A1=(1-X0)^2-2*E*X0*(1-Xz)^2;
A2=-2*Xz*(1-X0)^2+E*X0^2*(1-Xz)^2;  A3=Xz^2*(1-X0)^2;
Q=(27*A0^2*A3-9*A0*A1*A2+2*A1^3)/27/A0^3;
D=(4*A0*A2^3-18*A0*A1*A2*A3+4*A1^3*A3-A1^2*A2^2+27*A0^2*A3^2)/4/27/A0^4;
A=-Q/2+sqrt(D);                     B=-Q/2-sqrt(D);
Y=sign(A)*abs(A)^(1/3)+sign(B)*abs(B)^(1/3)-A1/3/A0;
r1=sqrt(abs(Y));
zeta=1/sqrt(1+1/r1^2);
% ****************************************************************************


