% ############################################################################
% elln.m    Elliptic or Cauer filter design.
%           Ws  : normalized stopband edge frequency (unity cutoff frequency)
%           Eps : ripple - factor
%           Z        : transfer function zeros are 0 +- j Z
%           C        : transfer function poles are C and conjugate of C
%           Wp       : magnitude of C
%           Q_factor : Q-factor of poles
% function [C,Z,Wp,Q_factor]=elln(N,Ws,Eps)
% ____________________________________________________________________________

function [C,Z,Wp,Q_factor]=elln(N,Ws,Eps)
   WS2   =Ws^2;               T   =sqrt(1-1/WS2);     EPS=Eps;
if     N==2
   W  =Ws*sqrt(1-T);       Z   =Ws/W;
   z  =ellc2(Ws,Eps);      C   =polzeta(W,Ws,z);
elseif N==3
   W  =ellz30(Ws);         Z   =Ws/W;                W =[0 W];
   z  =ellc3(Ws,Eps);      C   =polzeta(W,Ws,z);
else
   [R2,R3,Rest]=ellr2r3r(N);
   if Rest==1
      indW=1;WS(indW)=Ws;
      if R2>0
         for indR=1:R2
            indW=indW+1;  WS(indW)=elll2(WS(indW-1));
         end
      end
      if R3>0
         for indR=1:R3
            indW=indW+1;  WS(indW)=elll3(WS(indW-1));
         end
      end
      indw=indW-1;
      if R3>0
         W=ellz30(WS(indw));
         z  =ellc3(WS(indw),EPS);  EPS=sqrt(1/z/z-1);     indw=indw-1;
         if R3>1
            for indR=2:R3
               W=ellz33(W,WS(indw));
               z  =ellc3(WS(indw),EPS);  EPS=sqrt(1/z/z-1);    indw=indw-1;
            end
         end
      end
      if R2>0
         if R3==0
            W=WS(R2)*sqrt(1-sqrt(1-1/WS(R2)^2));  
            z  =ellc2(WS(R2),EPS);         EPS=sqrt(1/z/z-1);
            if R2>1
               indw=R2-1;
               for indR=1:R2-1
                  W=ellz2(W,WS(indw));
                  z  =ellc2(WS(indw),EPS);  EPS=sqrt(1/z/z-1);  indw=indw-1;
               end
            end
         else
            for indR=1:R2
               W=ellz2(W,WS(indw));
               z  =ellc2(WS(indw),EPS);  EPS=sqrt(1/z/z-1);  indw=indw-1;
            end
         end
      end
      if R2==0&R3>0
         Z =sort(Ws*ones(size(W))./W);       W=sort([0 W]);
      elseif R2>0&R3>0
         W1=WS(R2)*sqrt(1-sqrt(1-1/WS(R2)^2));  
         if R2>1
            indw=R2-1;
            for indR=1:R2-1
               W1=ellz2(W1,WS(indw));  indw=indw-1;
            end
         end
         W=sort([W1 W]);               Z=sort(Ws*ones(size(W))./W);
      elseif R2>0&R3==0
         W=sort(W);                    Z=sort(Ws*ones(size(W))./W);
      end
      C   =sort(polzeta(W,Ws,z));
   else
      disp(' N nije 2^i 3^j ')
   end
end
Wp=sqrt(real(C).^2+imag(C).^2);  Q_factor=-Wp./real(C)/2;
end
% ****************************************************************************
