function Z=ellz(N,Ws)
% ############################################################################
%
% ellz.m    W=ellz(N,Ws) finds the zeros of the Nth-order characteristic
%           function of the Cauer filter.
%           Ws is the  normalized stopband edge frequency (unity cutoff)
%           W=ellz is a function used to calculate a zeros of the 
%           Cauer filters of the order N= 5, 7, 10, 11, 13, 15, 17 (ellc.m).
%
%           M. Lutovac 6-18-1994
%
% function W=ellz30(Ws)
%
% ____________________________________________________________________________

if N==5 | N==7 | N==10 | N==11 | N==13 | N==14 | N==15 | N==17
   q=lmqws(Ws);
   K=sqrt(Ws*sqrt(q));
   if N==5
      a1=sqrt(5+sqrt(5)); a2=sqrt(5-sqrt(5)); a3=sqrt(5)+1; a4=sqrt(5)-1;
      d1=1-q^20-q^30; d2=q^2-q^12-q^42; d3=q-q^16-q^36; d4=q^4-q^9-q^49;
      W51=(a1*d1+a2*d2)/(2-4*q^25+a3*d3+a4*d4);
      W52=(a2*d1-a1*d2)/(2-4*q^25-a4*d3-a3*d4);
      Z=K*sqrt(2)*[W51 W52];
   elseif N==7
      s1=sin(pi/7); s2=sin(2*pi/7); s3=sin(3*pi/7); 
      c1=cos(pi/7); c2=cos(2*pi/7); c3=cos(3*pi/7); 
      d1=1-q^42;  d2=q^2-q^30;  d3=q^6-q^20;
      d4=1-2*q^49;d5=2*(q-q^36);  d6=2*(q^4-q^25);  d7=2*(q^9-q^16);
      W71=(d1*s3+d2*s2+d3*s1)/(d4+d5*c1+d6*c2+d7*c3);
      W72=(d1*s2-d2*s1-d3*s3)/(d4+d5*c3-d6*c1-d7*c2);
      W73=(d1*s1-d2*s3+d3*s2)/(d4-d5*c2-d6*c3+d7*c1);
      Z=2*K*[W71 W72 W73];
   elseif N==10
      s1=sqrt((2*sqrt(2)-sqrt(5+sqrt(5)))/(4*sqrt(2)));
      s3=sqrt((2*sqrt(2)-sqrt(5-sqrt(5)))/(4*sqrt(2)));  s5=sqrt(2)/2;
      s7=sqrt((2*sqrt(2)+sqrt(5-sqrt(5)))/(4*sqrt(2)));
      s9=sqrt((2*sqrt(2)+sqrt(5+sqrt(5)))/(4*sqrt(2)));
      c1=sqrt(5+sqrt(5))/2/sqrt(2);  c2=(1+sqrt(5))/4;
      c3=sqrt(5-sqrt(5))/2/sqrt(2);  c4=(-1+sqrt(5))/4;
      d1=(q^12-q^42);  d2=(q^20-q^30);  d3=2*(q^9-q^49);  d4=2*(q^16-q^36);
      d5=s5*q^6; d6=2*q*c1; d7=2*q*c3; d8=d4*c4+2*q^4*c2; d9=d4*c2+2*q^4*c4;
      W103=1/(sqrt(1+sqrt(1-(1/Ws^2))));
      W101=(s9+q^2*s7+d5+d1*s3+d2*s1)/(1+d6+d3*c3+d8);
      W102=(s7+q^2*s1-d5-d1*s9-d2*s3)/(1+d7-d3*c1-d9);
      W104=(s3-q^2*s9+d5+d1*s1-d2*s7)/(1-d7+d3*c1-d9);
      W105=(s1-q^2*s3+d5-d1*s7+d2*s9)/(1-d6-d3*c3+d8);
      Z=2*K*[W101 W102 0 W104 W105];
      Z(3)=W103;
   elseif N==11
      s1=sin(pi/11);  s2=sin(2*pi/11); s3=sin(3*pi/11); 
      s4=sin(4*pi/11);s5=sin(5*pi/11);
      c1=cos(pi/11);  c2=cos(2*pi/11); c3=cos(3*pi/11); 
      c4=cos(4*pi/11);c5=cos(5*pi/11);
      d1=2*(q^16-q^49);  d2=2*(q^25-q^36);  d3=q^20-q^42;
      d4=q^12; d5=2*q^9; d6=q^6; d7=q^2; d8=2*q^4;
      W111=(s5+d7*s4+d6*s3+d4*s2+d3*s1)/(1+2*q*c1+d8*c2+d5*c3+d1*c4+d2*c5);
      W112=(s4+d7*s1-d6*s2-d4*s5-d3*s3)/(1+2*q*c3-d8*c5-d5*c2-d1*c1-d2*c4);
      W113=(s3-d7*s2-d6*s4+d4*s1+d3*s5)/(1+2*q*c5-d8*c1-d5*c4+d1*c2+d2*c3);
      W114=(s2-d7*s5+d6*s1+d4*s3-d3*s4)/(1-2*q*c4-d8*c3+d5*c1-d1*c5-d2*c2);
      W115=(s1-d7*s3+d6*s5-d4*s4+d3*s2)/(1-2*q*c2+d8*c4+d5*c5-d1*c3+d2*c1);
      Z=2*K*[W111 W112 W113 W114 W115];
   elseif N==13
      s1=sin(pi/13);    s2=sin(2*pi/13);  s3=sin(3*pi/13); 
      s4=sin(4*pi/13);  s5=sin(5*pi/13);  s6=sin(6*pi/13);
      c1=cos(pi/13);    c2=cos(2*pi/13);  c3=cos(3*pi/13);
      c4=cos(4*pi/13);  c5=cos(5*pi/13);  c6=cos(6*pi/13);
      d1=2*(q^36-q^49); d2=q^2; d6=q^6; d12=q^12; d20=q^20; d30=q^30;
      d4=2*q^4; d9=2*q^9; d16=2*q^16; d25=2*q^25;
      W131=(s6+d2*s5+d6*s4+d12*s3+d20*s2+d30*s1)/...
         (1+2*q*c1+d4*c2+d9*c3+d16*c4+d25*c5+d1*c6);
      W132=(s5+d2*s2-d6*s1-d12*s4-d20*s6-d30*s3)/...
         (1+2*q*c3+d4*c6-d9*c4-d16*c1-d25*c2-d1*c5);
      W133=(s4-d2*s1-d6*s6-d12*s2+d20*s3+d30*s5)/...
         (1+2*q*c5-d4*c3-d9*c2+d16*c6+d25*c1+d1*c4);
      W134=(s3-d2*s4-d6*s2+d12*s5+d20*s1-d30*s6)/...
         (1-2*q*c6-d4*c1+d9*c5+d16*c2-d25*c4-d1*c3);
      W135=(s2-d2*s6+d6*s3+d12*s1-d20*s5+d30*s4)/...
         (1-2*q*c4-d4*c5+d9*c1-d16*c3-d25*c6+d1*c2);
      W136=(s1-d2*s3+d6*s5-d12*s6+d20*s4-d30*s2)/...
         (1-2*q*c2+d4*c4-d9*c6-d16*c5+d25*c3-d1*c1);
      Z=2*K*[W131 W132 W133 W134 W135 W136];
   elseif N==14
      s1=sin(pi/28);    s2=sin(3*pi/28);  s3=sin(5*pi/28); 
      s4=1/sqrt(2);     s5=sin(9*pi/28);  s6=sin(11*pi/28);  s7=sin(13*pi/28);
      c1=cos(pi/14);    c2=cos(pi/7);  c3=cos(3*pi/14);
      c4=cos(2*pi/7);  c5=cos(5*pi/14);  c6=cos(3*pi/7);
      d1=q^2; d2=q^6;  d3=q^12;  d4=q^20; d5=q^30; d42=q^42;
      d6=q^4; d7=q^9;  d8=q^25;  d9=q^36; d16=q^16;
      W141=(s7+d1*s6+d2*s5+s4*d3+d4*s3+d5*s2+d42*s1)/...
         (1+2*(q*c1+d6*c2+d7*c3+d16*c4+d8*c5+d9*c6));
      W142=(s6+d1*s3-d2*s1-s4*d3-d4*s7-d5*s5-d42*s2)/...
         (1+2*(q*c3+d6*c6-d7*c5-d16*c2-d8*c1-d9*c4));
      W143=(s5-d1*s1-d2*s6-s4*d3+d4*s2+d5*s7+d42*s3)/...
         (1+2*(q*c5-d6*c4-d7*c1-d16*c6+d8*c3+d9*c2));
      W144=1/(sqrt(1+sqrt(1-(1/Ws^2))));
      W145=(s3-d1*s7+d2*s2+s4*d3-d4*s6+d5*s1+d42*s5)/...
         (1-2*(q*c5+d6*c4-d7*c1+d16*c6+d8*c3-d9*c2));
      W146=(s2-d1*s5+d2*s7-s4*d3+d4*s1+d5*s3-d42*s6)/...
         (1-2*(q*c3-d6*c6-d7*c5+d16*c2-d8*c1+d9*c4));
      W147=(s1-d1*s2+d2*s3-s4*d3+d4*s5-d5*s6+d42*s7)/...
         (1-2*(q*c1-d6*c2+d7*c3-d16*c4+d8*c5-d9*c6));
      Z=2*K*[W141 W142 W143 0 W145 W146 W147];
      Z(4)=W144;
   elseif N==15
      s1=(sqrt(7-sqrt(5)-sqrt(30-6*sqrt(5))))/4;
      s2=(sqrt(7+sqrt(5)-sqrt(30+6*sqrt(5))))/4;
      s3=(sqrt(5-sqrt(5)))/(2*sqrt(2));
      s4=(sqrt(7-sqrt(5)+sqrt(30-6*sqrt(5))))/4;
      s5=sqrt(3)/2;      s6=(sqrt(5+sqrt(5)))/(2*sqrt(2));
      s7=(sqrt(7+sqrt(5)+sqrt(30+6*sqrt(5))))/4;
      c1=(sqrt(5)-1+sqrt(30+6*sqrt(5)))/8;
      c2=(1+sqrt(5)+sqrt(30-6*sqrt(5)))/8;
      c3=(1+sqrt(5))/4;  c4=(1-sqrt(5)+sqrt(30+6*sqrt(5)))/8;
      c6=(-1+sqrt(5))/4; c7=(-1-sqrt(5)+sqrt(30-6*sqrt(5)))/8;
      d1=q^2; d10=q^20; d15=q^25;  d6=q^6; d3=q^30; d16=q^16;
      d42=q^42;  d12=q^12; d36=q^36; d4=q^4; d9=q^9; d49=q^49;
      W151=(s7+d1*s6+s5*d6+d12*s4+d10*s3+d3*s2+d42*s1)/...
         (1+2*(q*c1+d4*c2+d9*c3+d16*c4+d15/2+d36*c6+d49*c7));
      W1525=ellz(5,Ws);
      W152=W1525(1);
      W153=ellz30(Ws);
      W154=(s4-d1*s3-s5*d6+d12*s2+d10*s6-d3*s1-d42*s7)/...
         (1+2*(q*c7-d4*c1-d9*c6+d16*c2+d15/2-d36*c3-d49*c4));
      W155=W1525(2);
      W156=(s2-d1*s6+s5*d6-d12*s1-d10*s3+d3*s7-d42*s4)/...
         (1-2*(q*c4+d4*c7-d9*c3+d16*c1-d15/2-d36*c6+d49*c2));
      W157=(s1-d1*s3+s5*d6-d12*s7+d10*s6-d3*s4+d42*s2)/...
         (1-2*(q*c2-d4*c4+d9*c6+d16*c7-d15/2+d36*c3-d49*c1));
      Z=2*K*[W151 0 0 W154 0 W156 W157];
      Z(2)=W152; Z(3)=W153; Z(5)=W155;
   elseif N==17
      s1=sin(pi/17);   s2=sin(2*pi/17);  s3=sin(3*pi/17);  s4=sin(4*pi/17);
      s5=sin(5*pi/17); s6=sin(6*pi/17);  s7=sin(7*pi/17);  s8=sin(8*pi/17);
      c1=cos(pi/17);   c2=cos(2*pi/17);  c3=cos(3*pi/17);  c4=cos(4*pi/17);
      c5=cos(5*pi/17); c6=cos(6*pi/17);  c7=cos(7*pi/17);  c8=cos(8*pi/17);
      d1=q^2; d2=q^6;  d3=q^12;  d4=q^20; d5=q^30; d42=q^42;
      d6=q^4; d7=q^9;  d8=q^25;  d9=q^36; d16=q^16;d49=q^49;
      W171=(s8+d1*s7+d2*s6+d3*s5+d4*s4+d5*s3+d42*s2)/...
         (1+2*(q*c1+d6*c2+d7*c3+d16*c4+d8*c5+d9*c6+d49*c7));
      W172=(s7+d1*s4+d2*s1-d3*s2-d4*s5-d5*s8-d42*s6)/...
         (1+2*(q*c3+d6*c6-d7*c8-d16*c5-d8*c2-d9*c1-d49*c4));
      W173=(s6+d1*s1-d2*s4-d3*s8-d4*s3+d5*s2+d42*s7)/...
         (1+2*(q*c5-d6*c7-d7*c2-d16*c3-d8*c8+d9*c4+d49*c1));
      W174=(s5-d1*s2-d2*s8-d3*s1+d4*s6+d5*s4-d42*s3)/...
         (1+2*(q*c7-d6*c3-d7*c4+d16*c6+d8*c1+d9*c8-d49*c2));
      W175=(s4-d1*s5-d2*s3+d3*s6+d4*s2-d5*s7-d42*s1)/...
         (1-2*(q*c8+d6*c1-d7*c7-d16*c2+d8*c6+d9*c3-d49*c5));
      W176=(s3-d1*s8+d2*s2+d3*s4-d4*s7+d5*s1+d42*s5)/...
         (1-2*(q*c6+d6*c5-d7*c1+d16*c7+d8*c4-d9*c2+d49*c8));
      W177=(s2-d1*s6+d2*s7-d3*s3-d4*s1+d5*s5-d42*s8)/...
         (1-2*(q*c4-d6*c8-d7*c5+d16*c1-d8*c3+d9*c7+d49*c6));
      W178=(s1-d1*s3+d2*s5-d3*s7+d4*s8-d5*s6+d42*s4)/...
         (1-2*(q*c2-d6*c4+d7*c6-d16*c8-d8*c7+d9*c5-d49*c3));
      Z=2*K*[W171 W172 W173 W174 W175 W176 W177 W178];
   end
else
   disp(' N = 5, 7, 10, 11, 13, 14, 15, 17 ')
end
% ****************************************************************************


