function W=ellz33(W1,Ws)
% ############################################################################
%
% ellz33.m  W=ellz33(W1,Ws) finds the zeros of the equation R_3(W,Ws) = W1 
%           where R_3(W,Ws) is the third-order characteristic function of
%           the Cauer filter, Ws is the normalized stopband edge frequency 
%           (with unity cutoff frequency) and W1 < 1.
%           ellz33.m returns the transfer function zeros of R_3N when W1
%           are the zeros of the Nth-order characteristic function, R_N. 
%           W=ellz33 is a function used to calculate the zeros of the 
%           Cauer filters of the order N= 9, 12, 18 (ellc.m).
%
%           M. Lutovac 6-18-1994
%
% function W=ellz33(W1,Ws)
% ____________________________________________________________________________

W=[];
for indW=1:length(W1)
   WS2=Ws^2;
   W3 =ellz30(Ws);            I2 =(Ws/W3)^2;
   a0 =(1-I2)/(I2-WS2);       a1 =-W1(indW)/I2/a0/6;
   a2 =-WS2/I2/6;             a3 =W1(indW)/a0/6;
   a0 =1/6;
   F  =acos((-3*a3+6*a1*a2-8*a1^3)*(-2*a2+4*a1^2)^(-3/2));
   y  =sqrt(-8*a2+16*a1^2);
   W=[W -2*a1+y*cos(F/3) 2*a1-y*cos((F+2*pi)/3) 2*a1-y*cos((F+4*pi)/3)];
   W=[W Ws/sqrt(I2)];
end
% ****************************************************************************


