% Example_10_3.m
%
%       Created for the book:                                                             
%   Ljiljana D. Milic,
%      Multirate Filtering for Digital Signal Processing: MATLAB Applications                     
%      Information Science Reference, Hershey, New York,  2009
%
% Design od the model filter
clear all, close all
g = firhalfband(22,0.4);         % Model filter design     
[G,fr] = freqz(g,1,1024,2);
figure (1)
plot(fr,20*log10(abs(G)));
xlabel('Frequency (Hz)'),ylabel('Gain(dB)'), title('Model filter')
%
% Design of the masking filters
ap = 0.1; as = 40;
delta_p = (10^(ap/20)-1)/(10^(ap/20)+1); % Passband ripple
delta_s = 10^(-as/20);                   % Stopband ripple
freq = [0,0.6,0.85,1]; m0 = [1,1,0,0]; w = [delta_p/delta_s,1]; % Seting design parameters for masking filter f0
f0 = firpm(18,freq,m0,w);     % Designing the masking filter f0
freq = [0,0.4,0.625,1]; m1=[1,1,0,0];  % Seting design parameters for masking filter f1
f1 = firpm(20,freq,m1,w);     % Designing the masking filter f1
[F0,fr] = freqz(f0,1,1024,2);
[F1,fr] = freqz(f1,1,1024,2);
figure (2)
plot(fr,20*log10(abs(F0)),fr,20*log10(abs(F1)),'--');
ylabel('Gain(dB)'),title('Masking filter')
f0=[0,f0,0];                          % Equalization of the masking filters' delays
%
% Generation of the model filter
gm = zeros(size(1:4*length(g)));        
gm([1:4:length(gm)]) = g;                   % Periodic model filter
[Gm,fr] = freqz(gm,1,1024,2);
figure (3)
plot(fr,20*log10(abs(Gm)));
ylabel('Gain(dB)'),xlabel('Frequency (Hz)')
title('Periodic model filter')
%
% Implementation of the wide-band filter according to Figure 10.8
d = zeros(size(gm)); d(45) = 1;          % Delay branch
gmc = d - gm;                            % Complementary periodic model filter
[Gmc,fr]=freqz(gmc,1,1024,2);
h = conv(gm,f0) + conv(gmc,f1);              % Impulse response of the overall filter
[H,Fr]=freqz(h,1,1024,2000);
figure (4)
plot(Fr,20*log10(abs(H))); 
xlabel('Frequency [Hz]'),ylabel('Gain, dB')
hold on
plot([0,650,650,1000],[0,0,-40,-40],'--')

%title('Wide-band filter')
axis([0,1000,-60,5])
g(1)=axes('Position',[0.20 0.55 0.40 0.07]);
plot(Fr(1:650),20*log10(abs(H(1:650)))),axis([0,600,-0.1,0.1])
figure (5)
subplot(3,1,1)
plot(fr,20*log10(abs(G))); ylabel('Gain, dB')
axis([0,1,-60,5])
subplot(3,1,2)
plot(fr,20*log10(abs(Gm)),fr,20*log10(abs(Gmc)),'--b'); ylabel('Gain, dB')
axis([0,1,-60,5])
subplot(3,1,3)
plot(fr,20*log10(abs(F0)),fr,20*log10(abs(F1)),'--b'); ylabel('Gain, dB')
axis([0,1,-60,5])
xlabel('\omega/\pi')

